{% extends change_form_template|default:"admin/cms/page/plugin/change_form.html" %}
{% load i18n %}

{% block content %}
    <p>
        {% translate "Display content only for authenticated users or users with data hangovered from provider." %}
    </p>
    {{ block.super }}
{% endblock %}