# Generated by Django 2.2.13 on 2021-03-26 11:38

from django.db import migrations, models
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_oidc', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='oidchandoverdata',
            name='button_label',
            field=models.CharField(blank=True, help_text='Button text for unlogged in user.', max_length=80, null=True, verbose_name='Button label'),
        ),
        migrations.AddField(
            model_name='oidclogin',
            name='button_label',
            field=models.CharField(blank=True, help_text='Button text for unlogged in user.', max_length=80, null=True, verbose_name='Button label'),
        ),
        migrations.AlterField(
            model_name='oidcdisplaydedicatedcontent',
            name='conditions',
            field=models.CharField(blank=True, choices=[('only_authenticated_user', 'Only authenticated user'), ('only_user_is_staff', 'Only user is staff'), ('email_verified', 'Email verified'), ('validated_account_to_submit_order', 'Validated account to submit order')], help_text='Show content only if the given conditions are met.', max_length=255, null=True, verbose_name='Conditions'),
        ),
        migrations.AlterField(
            model_name='oidchandoverdata',
            name='authorization_prompt',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('none', 'No interaction'), ('login', 'Force login'), ('consent', 'Force consent with handovered data')], help_text='Prompt for user at authorization. "No interaction" cannot be combined with others.', max_length=255, null=True, verbose_name='Prompt'),
        ),
        migrations.AlterField(
            model_name='oidclogin',
            name='authorization_prompt',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('none', 'No interaction'), ('login', 'Force login'), ('consent', 'Force consent with handovered data')], help_text='Prompt for user at authorization. "No interaction" cannot be combined with others.', max_length=255, null=True, verbose_name='Prompt'),
        ),
    ]
